@@ -2,15 +2,18 @@
 This module interfaces with PROJ to produce a pythonic interface
 to the coordinate reference system (CRS) information.
 """
+# pylint: disable=too-many-lines
 import json
 import re
 import threading
 import warnings
-from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
+from abc import abstractmethod
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
-from pyproj._crs import (  # noqa
+from pyproj._crs import (
     _CRS,
     AreaOfUse,
+    AuthorityMatchInfo,
     Axis,
     CoordinateOperation,
     CoordinateSystem,
@@ -46,9 +49,12 @@ class CRSLocal(threading.local):
 
     def __init__(self):
         self.crs = None  # Initialises in each thread
+        super().__init__()
 
 
 def _prepare_from_dict(projparams: dict, allow_json: bool = True) -> str:
+    if not isinstance(projparams, dict):
+        raise CRSError("CRS input is not a dict")
     # check if it is a PROJ JSON dict
     if "proj" not in projparams and "init" not in projparams and allow_json:
         return json.dumps(projparams)
@@ -68,54 +74,61 @@ def _prepare_from_dict(projparams: dict, allow_json: bool = True) -> str:
     return _prepare_from_string(" ".join(pjargs))
 
 
+def _prepare_from_proj_string(in_crs_string: str) -> str:
+    in_crs_string = re.sub(r"[\s+]?=[\s+]?", "=", in_crs_string.lstrip())
+    # make sure the projection starts with +proj or +init
+    starting_params = ("+init", "+proj", "init", "proj")
+    if not in_crs_string.startswith(starting_params):
+        kvpairs: List[str] = []
+        first_item_inserted = False
+        for kvpair in in_crs_string.split():
+            if not first_item_inserted and (kvpair.startswith(starting_params)):
+                kvpairs.insert(0, kvpair)
+                first_item_inserted = True
+            else:
+                kvpairs.append(kvpair)
+        in_crs_string = " ".join(kvpairs)
+
+    # make sure it is the CRS type
+    if "type=crs" not in in_crs_string:
+        if "+" in in_crs_string:
+            in_crs_string += " +type=crs"
+        else:
+            in_crs_string += " type=crs"
+
+    # look for EPSG, replace with epsg (EPSG only works
+    # on case-insensitive filesystems).
+    in_crs_string = in_crs_string.replace("+init=EPSG", "+init=epsg").strip()
+    if in_crs_string.startswith(("+init", "init")):
+        warnings.warn(
+            "'+init=<authority>:<code>' syntax is deprecated. "
+            "'<authority>:<code>' is the preferred initialization method. "
+            "When making the change, be mindful of axis order changes: "
+            "https://pyproj4.github.io/pyproj/stable/gotchas.html"
+            "#axis-order-changes-in-proj-6",
+            FutureWarning,
+            stacklevel=2,
+        )
+    return in_crs_string
+
+
 def _prepare_from_string(in_crs_string: str) -> str:
+    if not isinstance(in_crs_string, str):
+        raise CRSError("CRS input is not a string")
     if not in_crs_string:
-        raise CRSError(f"CRS is empty or invalid: {in_crs_string!r}")
-    elif "{" in in_crs_string:
+        raise CRSError(f"CRS string is empty or invalid: {in_crs_string!r}")
+    if "{" in in_crs_string:
         # may be json, try to decode it
         try:
             crs_dict = json.loads(in_crs_string, strict=False)
-        except ValueError:
-            raise CRSError("CRS appears to be JSON but is not valid")
+        except ValueError as err:
+            raise CRSError("CRS appears to be JSON but is not valid") from err
 
         if not crs_dict:
             raise CRSError("CRS is empty JSON")
-        return _prepare_from_dict(crs_dict)
+        in_crs_string = _prepare_from_dict(crs_dict)
     elif is_proj(in_crs_string):
-        in_crs_string = re.sub(r"[\s+]?=[\s+]?", "=", in_crs_string.lstrip())
-        # make sure the projection starts with +proj or +init
-        starting_params = ("+init", "+proj", "init", "proj")
-        if not in_crs_string.startswith(starting_params):
-            kvpairs = []  # type: List[str]
-            first_item_inserted = False
-            for kvpair in in_crs_string.split():
-                if not first_item_inserted and (kvpair.startswith(starting_params)):
-                    kvpairs.insert(0, kvpair)
-                    first_item_inserted = True
-                else:
-                    kvpairs.append(kvpair)
-            in_crs_string = " ".join(kvpairs)
-
-        # make sure it is the CRS type
-        if "type=crs" not in in_crs_string:
-            if "+" in in_crs_string:
-                in_crs_string += " +type=crs"
-            else:
-                in_crs_string += " type=crs"
-
-        # look for EPSG, replace with epsg (EPSG only works
-        # on case-insensitive filesystems).
-        in_crs_string = in_crs_string.replace("+init=EPSG", "+init=epsg").strip()
-        if in_crs_string.startswith(("+init", "init")):
-            warnings.warn(
-                "'+init=<authority>:<code>' syntax is deprecated. "
-                "'<authority>:<code>' is the preferred initialization method. "
-                "When making the change, be mindful of axis order changes: "
-                "https://pyproj4.github.io/pyproj/stable/gotchas.html"
-                "#axis-order-changes-in-proj-6",
-                FutureWarning,
-                stacklevel=2,
-            )
+        in_crs_string = _prepare_from_proj_string(in_crs_string)
     return in_crs_string
 
 
@@ -321,8 +334,8 @@ class CRS:
             self._local.crs = _CRS(self.srs)
         return self._local.crs
 
-    @staticmethod
-    def from_authority(auth_name: str, code: Union[str, int]) -> "CRS":
+    @classmethod
+    def from_authority(cls, auth_name: str, code: Union[str, int]) -> "CRS":
         """
         .. versionadded:: 2.2.0
 
@@ -339,10 +352,10 @@ class CRS:
         -------
         CRS
         """
-        return CRS(_prepare_from_authority(auth_name, code))
+        return cls.from_user_input(_prepare_from_authority(auth_name, code))
 
-    @staticmethod
-    def from_epsg(code: Union[str, int]) -> "CRS":
+    @classmethod
+    def from_epsg(cls, code: Union[str, int]) -> "CRS":
         """Make a CRS from an EPSG code
 
         Parameters
@@ -354,10 +367,10 @@ class CRS:
         -------
         CRS
         """
-        return CRS(_prepare_from_epsg(code))
+        return cls.from_user_input(_prepare_from_epsg(code))
 
-    @staticmethod
-    def from_proj4(in_proj_string: str) -> "CRS":
+    @classmethod
+    def from_proj4(cls, in_proj_string: str) -> "CRS":
         """
         .. versionadded:: 2.2.0
 
@@ -374,10 +387,10 @@ class CRS:
         """
         if not is_proj(in_proj_string):
             raise CRSError(f"Invalid PROJ string: {in_proj_string}")
-        return CRS(_prepare_from_string(in_proj_string))
+        return cls.from_user_input(_prepare_from_proj_string(in_proj_string))
 
-    @staticmethod
-    def from_wkt(in_wkt_string: str) -> "CRS":
+    @classmethod
+    def from_wkt(cls, in_wkt_string: str) -> "CRS":
         """
         .. versionadded:: 2.2.0
 
@@ -394,10 +407,10 @@ class CRS:
         """
         if not is_wkt(in_wkt_string):
             raise CRSError(f"Invalid WKT string: {in_wkt_string}")
-        return CRS(_prepare_from_string(in_wkt_string))
+        return cls.from_user_input(_prepare_from_string(in_wkt_string))
 
-    @staticmethod
-    def from_string(in_crs_string: str) -> "CRS":
+    @classmethod
+    def from_string(cls, in_crs_string: str) -> "CRS":
         """
         Make a CRS from:
 
@@ -416,7 +429,7 @@ class CRS:
         -------
         CRS
         """
-        return CRS(_prepare_from_string(in_crs_string))
+        return cls.from_user_input(_prepare_from_string(in_crs_string))
 
     def to_string(self) -> str:
         """
@@ -437,8 +450,8 @@ class CRS:
             return ":".join(auth_info)
         return self.srs
 
-    @staticmethod
-    def from_user_input(value: Any, **kwargs) -> "CRS":
+    @classmethod
+    def from_user_input(cls, value: Any, **kwargs) -> "CRS":
         """
         Initialize a CRS class instance with:
           - PROJ string
@@ -461,9 +474,9 @@ class CRS:
         -------
         CRS
         """
-        if isinstance(value, CRS):
+        if isinstance(value, cls):
             return value
-        return CRS(value, **kwargs)
+        return cls(value, **kwargs)
 
     def get_geod(self) -> Optional[Geod]:
         """
@@ -480,8 +493,8 @@ class CRS:
             b=self.ellipsoid.semi_minor_metre,
         )
 
-    @staticmethod
-    def from_dict(proj_dict: dict) -> "CRS":
+    @classmethod
+    def from_dict(cls, proj_dict: dict) -> "CRS":
         """
         .. versionadded:: 2.2.0
 
@@ -496,10 +509,10 @@ class CRS:
         -------
         CRS
         """
-        return CRS(_prepare_from_dict(proj_dict))
+        return cls.from_user_input(_prepare_from_dict(proj_dict))
 
-    @staticmethod
-    def from_json(crs_json: str) -> "CRS":
+    @classmethod
+    def from_json(cls, crs_json: str) -> "CRS":
         """
         .. versionadded:: 2.4.0
 
@@ -514,10 +527,10 @@ class CRS:
         -------
         CRS
         """
-        return CRS.from_json_dict(_load_proj_json(crs_json))
+        return cls.from_user_input(_load_proj_json(crs_json))
 
-    @staticmethod
-    def from_json_dict(crs_dict: dict) -> "CRS":
+    @classmethod
+    def from_json_dict(cls, crs_dict: dict) -> "CRS":
         """
         .. versionadded:: 2.4.0
 
@@ -532,7 +545,7 @@ class CRS:
         -------
         CRS
         """
-        return CRS(json.dumps(crs_dict))
+        return cls.from_user_input(json.dumps(crs_dict))
 
     def to_dict(self) -> dict:
         """
@@ -554,7 +567,7 @@ class CRS:
         def parse(val):
             if val.lower() == "true":
                 return True
-            elif val.lower() == "false":
+            if val.lower() == "false":
                 return False
             try:
                 return int(val)
@@ -595,8 +608,8 @@ class CRS:
         wkt_version: str or pyproj.enums.WktVersion
             Version of WKT supported by CRS.to_wkt.
             Default is :attr:`pyproj.enums.WktVersion.WKT2_2019`.
-        errcheck: bool, optional
-            If True, will warn when parameters are ignored. Defaults to False.
+        errcheck: bool, default=False
+            If True, will warn when parameters are ignored.
 
         Returns
         -------
@@ -604,30 +617,35 @@ class CRS:
             CF-1.8 version of the projection.
 
         """
-        cf_dict = {"crs_wkt": self.to_wkt(wkt_version)}  # type: Dict[str, Any]
+        # pylint: disable=too-many-branches
+        cf_dict: Dict[str, Any] = {"crs_wkt": self.to_wkt(wkt_version)}
 
         # handle bound CRS
         if (
             self.is_bound
             and self.coordinate_operation
             and self.coordinate_operation.towgs84
+            and self.source_crs
         ):
-            sub_cf = self.source_crs.to_cf(errcheck=errcheck)  # type: ignore
+            sub_cf: Dict[str, Any] = self.source_crs.to_cf(
+                wkt_version=wkt_version,
+                errcheck=errcheck,
+            )
             sub_cf.pop("crs_wkt")
             cf_dict.update(sub_cf)
             cf_dict["towgs84"] = self.coordinate_operation.towgs84
             return cf_dict
 
         # handle compound CRS
-        elif self.sub_crs_list:
+        if self.is_compound:
             for sub_crs in self.sub_crs_list:
-                sub_cf = sub_crs.to_cf(errcheck=errcheck)
+                sub_cf = sub_crs.to_cf(wkt_version=wkt_version, errcheck=errcheck)
                 sub_cf.pop("crs_wkt")
                 cf_dict.update(sub_cf)
             return cf_dict
 
         # handle vertical CRS
-        elif self.is_vertical:
+        if self.is_vertical:
             vert_json = self.to_json_dict()
             if "geoid_model" in vert_json:
                 cf_dict["geoid_name"] = vert_json["geoid_model"]["name"]
@@ -708,6 +726,8 @@ class CRS:
 
         .. versionadded:: 3.0.0 ellipsoidal_cs, cartesian_cs, vertical_cs
 
+        .. deprecated:: 3.2.0 errcheck
+
         This converts a Climate and Forecast (CF) Grid Mapping Version 1.8
         dict to a :obj:`pyproj.crs.CRS` object.
 
@@ -729,18 +749,26 @@ class CRS:
             Input to create a Vertical Coordinate System accepted by
             :meth:`pyproj.crs.CoordinateSystem.from_user_input`
             or :class:`pyproj.crs.coordinate_system.VerticalCS`
-        errcheck: bool, optional
+        errcheck: bool, default=False
             This parameter is for backwards compatibility with the old version.
-            It currently does nothing when True or False.
+            It currently does nothing when True or False. DEPRECATED.
 
         Returns
         -------
         CRS
         """
+        # pylint: disable=too-many-branches
+        if errcheck:
+            warnings.warn(
+                "errcheck is deprecated as it does nothing.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
         unknown_names = ("unknown", "undefined")
         if "crs_wkt" in in_cf:
             return CRS(in_cf["crs_wkt"])
-        elif "spatial_ref" in in_cf:  # for previous supported WKT key
+        if "spatial_ref" in in_cf:  # for previous supported WKT key
             return CRS(in_cf["spatial_ref"])
 
         grid_mapping_name = in_cf.get("grid_mapping_name")
@@ -752,19 +780,19 @@ class CRS:
 
         # build geographic CRS
         try:
-            geographic_conversion_method = _GEOGRAPHIC_GRID_MAPPING_NAME_MAP[
-                grid_mapping_name
-            ]  # type: Optional[Callable]
+            geographic_conversion_method: Optional[
+                Callable
+            ] = _GEOGRAPHIC_GRID_MAPPING_NAME_MAP[grid_mapping_name]
         except KeyError:
             geographic_conversion_method = None
 
         geographic_crs_name = in_cf.get("geographic_crs_name")
         if datum:
-            geographic_crs = GeographicCRS(
+            geographic_crs: CRS = GeographicCRS(
                 name=geographic_crs_name or "undefined",
                 datum=datum,
                 ellipsoidal_cs=ellipsoidal_cs,
-            )  # type: CRS
+            )
         elif geographic_crs_name and geographic_crs_name not in unknown_names:
             geographic_crs = CRS(geographic_crs_name)
             if ellipsoidal_cs is not None:
@@ -788,7 +816,9 @@ class CRS:
         try:
             conversion_method = _GRID_MAPPING_NAME_MAP[grid_mapping_name]
         except KeyError:
-            raise CRSError(f"Unsupported grid mapping name: {grid_mapping_name}")
+            raise CRSError(
+                f"Unsupported grid mapping name: {grid_mapping_name}"
+            ) from None
         projected_crs = ProjectedCRS(
             name=in_cf.get("projected_crs_name", "undefined"),
             conversion=conversion_method(in_cf),
@@ -884,7 +914,7 @@ class CRS:
                 cf_axis_list.extend(sub_crs.cs_to_cf())
         return cf_axis_list
 
-    def is_exact_same(self, other: Any, ignore_axis_order: bool = False) -> bool:
+    def is_exact_same(self, other: Any) -> bool:
         """
         Check if the CRS objects are the exact same.
 
@@ -918,9 +948,8 @@ class CRS:
             Check if the other object is equivalent to this object.
             If the other object is not a CRS, it will try to create one.
             On Failure, it will return False.
-        ignore_axis_order: bool, optional
+        ignore_axis_order: bool, default=False
             If True, it will compare the CRS class and ignore the axis order.
-            Default is False.
 
         Returns
         -------
@@ -943,7 +972,11 @@ class CRS:
             The the geodeticCRS / geographicCRS from the CRS.
 
         """
-        return None if self._crs.geodetic_crs is None else CRS(self._crs.geodetic_crs)
+        return (
+            None
+            if self._crs.geodetic_crs is None
+            else self.__class__(self._crs.geodetic_crs)
+        )
 
     @property
     def source_crs(self) -> Optional["CRS"]:
@@ -955,7 +988,11 @@ class CRS:
         -------
         CRS
         """
-        return None if self._crs.source_crs is None else CRS(self._crs.source_crs)
+        return (
+            None
+            if self._crs.source_crs is None
+            else self.__class__(self._crs.source_crs)
+        )
 
     @property
     def target_crs(self) -> Optional["CRS"]:
@@ -968,7 +1005,11 @@ class CRS:
             The hub CRS of a BoundCRS or the target CRS of a CoordinateOperation.
 
         """
-        return None if self._crs.target_crs is None else CRS(self._crs.target_crs)
+        return (
+            None
+            if self._crs.target_crs is None
+            else self.__class__(self._crs.target_crs)
+        )
 
     @property
     def sub_crs_list(self) -> List["CRS"]:
@@ -979,7 +1020,7 @@ class CRS:
         -------
         List[CRS]
         """
-        return [CRS(sub_crs) for sub_crs in self._crs.sub_crs_list]
+        return [self.__class__(sub_crs) for sub_crs in self._crs.sub_crs_list]
 
     @property
     def utm_zone(self) -> Optional[str]:
@@ -995,7 +1036,7 @@ class CRS:
         """
         if self.is_bound and self.source_crs:
             return self.source_crs.utm_zone
-        elif self.sub_crs_list:
+        if self.sub_crs_list:
             for sub_crs in self.sub_crs_list:
                 if sub_crs.utm_zone:
                     return sub_crs.utm_zone
@@ -1012,8 +1053,7 @@ class CRS:
         Returns
         -------
         str:
-            The name of the CRS (from `proj_get_name <https://proj.org/
-            development/reference/functions.html#_CPPv313proj_get_namePK2PJ>`_).
+            The name of the CRS (from :cpp:func:`proj_get_name`).
         """
         return self._crs.name
 
@@ -1151,11 +1191,11 @@ class CRS:
 
         Parameters
         ----------
-        version: pyproj.enums.WktVersion
+        version: pyproj.enums.WktVersion, optional
             The version of the WKT output.
             Default is :attr:`pyproj.enums.WktVersion.WKT2_2019`.
-        pretty: bool
-            If True, it will set the output to be a multiline string. Defaults to False.
+        pretty: bool, default=False
+            If True, it will set the output to be a multiline string.
 
         Returns
         -------
@@ -1171,10 +1211,10 @@ class CRS:
 
         Parameters
         ----------
-        pretty: bool
-            If True, it will set the output to be a multiline string. Defaults to False.
-        indentation: int
-            If pretty is True, it will set the width of the indentation. Default is 2.
+        pretty: bool, default=False
+            If True, it will set the output to be a multiline string.
+        indentation: int, default=2
+            If pretty is True, it will set the width of the indentation.
 
         Returns
         -------
@@ -1240,8 +1280,8 @@ class CRS:
 
         Parameters
         ----------
-        min_confidence: int, optional
-            A value between 0-100 where 100 is the most confident. Default is 70.
+        min_confidence: int, default=70
+            A value between 0-100 where 100 is the most confident.
             :ref:`min_confidence`
 
 
@@ -1281,8 +1321,8 @@ class CRS:
         ----------
         auth_name: str, optional
             The name of the authority to filter by.
-        min_confidence: int, optional
-            A value between 0-100 where 100 is the most confident. Default is 70.
+        min_confidence: int, default=70
+            A value between 0-100 where 100 is the most confident.
             :ref:`min_confidence`
 
         Returns
@@ -1294,6 +1334,50 @@ class CRS:
             auth_name=auth_name, min_confidence=min_confidence
         )
 
+    def list_authority(
+        self, auth_name: Optional[str] = None, min_confidence: int = 70
+    ) -> List[AuthorityMatchInfo]:
+        """
+        .. versionadded:: 3.2.0
+
+        Return the authority names and codes best matching the CRS.
+
+        Example:
+
+        >>> from pyproj import CRS
+        >>> ccs = CRS("epsg:4328")
+        >>> ccs.list_authority()
+        [AuthorityMatchInfo(auth_name='EPSG', code='4326', confidence=100)]
+
+        If the CRS is bound, you can get an authority from
+        the source CRS:
+
+        >>> from pyproj import CRS
+        >>> ccs = CRS("+proj=geocent +datum=WGS84 +towgs84=0,0,0")
+        >>> ccs.list_authority()
+        []
+        >>> ccs.source_crs.list_authority()
+        [AuthorityMatchInfo(auth_name='EPSG', code='4978', confidence=70)]
+        >>> ccs == CRS.from_authorty('EPSG', '4978')
+        False
+
+        Parameters
+        ----------
+        auth_name: str, optional
+            The name of the authority to filter by.
+        min_confidence: int, default=70
+            A value between 0-100 where 100 is the most confident.
+            :ref:`min_confidence`
+
+        Returns
+        -------
+        List[AuthorityMatchInfo]:
+            List of authority matches for the CRS.
+        """
+        return self._crs.list_authority(
+            auth_name=auth_name, min_confidence=min_confidence
+        )
+
     def to_3d(self, name: Optional[str] = None) -> "CRS":
         """
         .. versionadded:: 3.1.0
@@ -1314,7 +1398,7 @@ class CRS:
         -------
         CRS
         """
-        return CRS(self._crs.to_3d(name=name))
+        return self.__class__(self._crs.to_3d(name=name))
 
     @property
     def is_geographic(self) -> bool:
@@ -1410,12 +1494,27 @@ class CRS:
         """
         return self._crs.is_geocentric
 
+    @property
+    def is_derived(self):
+        """
+        .. versionadded:: 3.2.0
+
+        Returns
+        -------
+        bool:
+            True if CRS is a Derived CRS.
+        """
+        return self._crs.is_derived
+
     def __eq__(self, other: Any) -> bool:
         return self.equals(other)
 
-    def __reduce__(self) -> Tuple[Type["CRS"], Tuple[str]]:
-        """special method that allows CRS instance to be pickled"""
-        return self.__class__, (self.srs,)
+    def __getstate__(self) -> Dict[str, str]:
+        return {"srs": self.srs}
+
+    def __setstate__(self, state: Dict[str, Any]):
+        self.__dict__.update(state)
+        self._local = CRSLocal()
 
     def __hash__(self) -> int:
         return hash(self.to_wkt())
@@ -1425,7 +1524,7 @@ class CRS:
 
     def __repr__(self) -> str:
         # get axis information
-        axis_info_list = []  # type: List[str]
+        axis_info_list: List[str] = []
         for axis in self.axis_info:
             axis_info_list.extend(["- ", str(axis), "\n"])
         axis_info_str = "".join(axis_info_list)
@@ -1455,8 +1554,8 @@ class CRS:
                     "Coordinate Operation:\n",
                     "- name: ",
                     str(self.coordinate_operation),
-                    "\n" "- method: ",
-                    str(self.coordinate_operation.method_name),
+                    "\n- method: ",
+                    self.coordinate_operation.method_name,
                     "\n",
                 ]
             )
@@ -1481,13 +1580,147 @@ class CRS:
         )
 
 
-class GeographicCRS(CRS):
+class CustomConstructorCRS(CRS):
+    """
+    This class is a base class for CRS classes
+    that use a different constructor than the main CRS class.
+
+    .. versionadded:: 3.2.0
+
+    See: https://github.com/pyproj4/pyproj/issues/847
+    """
+
+    @property
+    @abstractmethod
+    def _expected_types(self) -> Tuple[str, ...]:
+        """
+        These are the type names of the CRS class
+        that are expected when using the from_* methods.
+        """
+        raise NotImplementedError
+
+    def _check_type(self):
+        """
+        This validates that the type of the CRS is expected
+        when using the from_* methods.
+        """
+        if self.type_name not in self._expected_types:
+            raise CRSError(
+                f"Invalid type {self.type_name}. Expected {self._expected_types}."
+            )
+
+    @classmethod
+    def from_user_input(cls, value: Any, **kwargs) -> "CRS":
+        """
+        Initialize a CRS class instance with:
+          - PROJ string
+          - Dictionary of PROJ parameters
+          - PROJ keyword arguments for parameters
+          - JSON string with PROJ parameters
+          - CRS WKT string
+          - An authority string [i.e. 'epsg:4326']
+          - An EPSG integer code [i.e. 4326]
+          - A tuple of ("auth_name": "auth_code") [i.e ('epsg', '4326')]
+          - An object with a `to_wkt` method.
+          - A :class:`pyproj.crs.CRS` class
+
+        Parameters
+        ----------
+        value : obj
+            A Python int, dict, or str.
+
+        Returns
+        -------
+        CRS
+        """
+        if isinstance(value, cls):
+            return value
+        crs = cls.__new__(cls)
+        super(CustomConstructorCRS, crs).__init__(value, **kwargs)
+        crs._check_type()
+        return crs
+
+    @property
+    def geodetic_crs(self) -> Optional["CRS"]:
+        """
+        .. versionadded:: 2.2.0
+
+        Returns
+        -------
+        CRS:
+            The the geodeticCRS / geographicCRS from the CRS.
+
+        """
+        return None if self._crs.geodetic_crs is None else CRS(self._crs.geodetic_crs)
+
+    @property
+    def source_crs(self) -> Optional["CRS"]:
+        """
+        The the base CRS of a BoundCRS or a DerivedCRS/ProjectedCRS,
+        or the source CRS of a CoordinateOperation.
+
+        Returns
+        -------
+        CRS
+        """
+        return None if self._crs.source_crs is None else CRS(self._crs.source_crs)
+
+    @property
+    def target_crs(self) -> Optional["CRS"]:
+        """
+        .. versionadded:: 2.2.0
+
+        Returns
+        -------
+        CRS:
+            The hub CRS of a BoundCRS or the target CRS of a CoordinateOperation.
+
+        """
+        return None if self._crs.target_crs is None else CRS(self._crs.target_crs)
+
+    @property
+    def sub_crs_list(self) -> List["CRS"]:
+        """
+        If the CRS is a compound CRS, it will return a list of sub CRS objects.
+
+        Returns
+        -------
+        List[CRS]
+        """
+        return [CRS(sub_crs) for sub_crs in self._crs.sub_crs_list]
+
+    def to_3d(self, name: Optional[str] = None) -> "CRS":
+        """
+        .. versionadded:: 3.1.0
+
+        Convert the current CRS to the 3D version if it makes sense.
+
+        New vertical axis attributes:
+          - ellipsoidal height
+          - oriented upwards
+          - metre units
+
+        Parameters
+        ----------
+        name: str, optional
+            CRS name. Defaults to use the name of the original CRS.
+
+        Returns
+        -------
+        CRS
+        """
+        return CRS(self._crs.to_3d(name=name))
+
+
+class GeographicCRS(CustomConstructorCRS):
     """
     .. versionadded:: 2.5.0
 
     This class is for building a Geographic CRS
     """
 
+    _expected_types = ("Geographic CRS", "Geographic 2D CRS", "Geographic 3D CRS")
+
     def __init__(
         self,
         name: str = "undefined",
@@ -1497,9 +1730,9 @@ class GeographicCRS(CRS):
         """
         Parameters
         ----------
-        name: str, optional
-            Name of the CRS. Default is undefined.
-        datum: Any, optional
+        name: str, default="undefined"
+            Name of the CRS.
+        datum: Any, default="urn:ogc:def:datum:EPSG::6326"
             Anything accepted by :meth:`pyproj.crs.Datum.from_user_input` or
             a :class:`pyproj.crs.datum.CustomDatum`.
         ellipsoidal_cs: Any, optional
@@ -1519,13 +1752,19 @@ class GeographicCRS(CRS):
         super().__init__(geographic_crs_json)
 
 
-class DerivedGeographicCRS(CRS):
+class DerivedGeographicCRS(CustomConstructorCRS):
     """
     .. versionadded:: 2.5.0
 
     This class is for building a Derived Geographic CRS
     """
 
+    _expected_types = (
+        "Derived Geographic CRS",
+        "Derived Geographic 2D CRS",
+        "Derived Geographic 3D CRS",
+    )
+
     def __init__(
         self,
         base_crs: Any,
@@ -1546,8 +1785,8 @@ class DerivedGeographicCRS(CRS):
             Input to create an Ellipsoidal Coordinate System.
             Anything accepted by :meth:`pyproj.crs.CoordinateSystem.from_user_input`
             or an Ellipsoidal Coordinate System created from :ref:`coordinate_system`.
-        name: str, optional
-            Name of the CRS. Default is undefined.
+        name: str, default="undefined"
+            Name of the CRS.
         """
         derived_geographic_crs_json = {
             "$schema": "https://proj.org/schemas/v0.2/projjson.schema.json",
@@ -1564,13 +1803,70 @@ class DerivedGeographicCRS(CRS):
         super().__init__(derived_geographic_crs_json)
 
 
-class ProjectedCRS(CRS):
+class GeocentricCRS(CustomConstructorCRS):
+    """
+    .. versionadded:: 3.2.0
+
+    This class is for building a Geocentric CRS
+    """
+
+    _expected_types = ("Geocentric CRS",)
+
+    def __init__(
+        self,
+        name: str = "undefined",
+        datum: Any = "urn:ogc:def:datum:EPSG::6326",
+    ) -> None:
+        """
+        Parameters
+        ----------
+        name: str, default="undefined"
+            Name of the CRS.
+        datum: Any, default="urn:ogc:def:datum:EPSG::6326"
+            Anything accepted by :meth:`pyproj.crs.Datum.from_user_input` or
+            a :class:`pyproj.crs.datum.CustomDatum`.
+        """
+        geocentric_crs_json = {
+            "$schema": ("https://proj.org/schemas/v0.2/projjson.schema.json"),
+            "type": "GeodeticCRS",
+            "name": name,
+            "datum": Datum.from_user_input(datum).to_json_dict(),
+            "coordinate_system": {
+                "subtype": "Cartesian",
+                "axis": [
+                    {
+                        "name": "Geocentric X",
+                        "abbreviation": "X",
+                        "direction": "geocentricX",
+                        "unit": "metre",
+                    },
+                    {
+                        "name": "Geocentric Y",
+                        "abbreviation": "Y",
+                        "direction": "geocentricY",
+                        "unit": "metre",
+                    },
+                    {
+                        "name": "Geocentric Z",
+                        "abbreviation": "Z",
+                        "direction": "geocentricZ",
+                        "unit": "metre",
+                    },
+                ],
+            },
+        }
+        super().__init__(geocentric_crs_json)
+
+
+class ProjectedCRS(CustomConstructorCRS):
     """
     .. versionadded:: 2.5.0
 
     This class is for building a Projected CRS.
     """
 
+    _expected_types = ("Projected CRS", "Derived Projected CRS")
+
     def __init__(
         self,
         conversion: Any,
@@ -1611,7 +1907,7 @@ class ProjectedCRS(CRS):
         super().__init__(proj_crs_json)
 
 
-class VerticalCRS(CRS):
+class VerticalCRS(CustomConstructorCRS):
     """
     .. versionadded:: 2.5.0
 
@@ -1621,6 +1917,8 @@ class VerticalCRS(CRS):
 
     """
 
+    _expected_types = ("Vertical CRS",)
+
     def __init__(
         self,
         name: str,
@@ -1657,13 +1955,15 @@ class VerticalCRS(CRS):
         super().__init__(vert_crs_json)
 
 
-class CompoundCRS(CRS):
+class CompoundCRS(CustomConstructorCRS):
     """
     .. versionadded:: 2.5.0
 
     This class is for building a Compound CRS.
     """
 
+    _expected_types = ("Compound CRS",)
+
     def __init__(self, name: str, components: List[Any]) -> None:
         """
         Parameters
@@ -1687,13 +1987,15 @@ class CompoundCRS(CRS):
         super().__init__(compound_crs_json)
 
 
-class BoundCRS(CRS):
+class BoundCRS(CustomConstructorCRS):
     """
     .. versionadded:: 2.5.0
 
     This class is for building a Bound CRS.
     """
 
+    _expected_types = ("Bound CRS",)
+
     def __init__(self, source_crs: Any, target_crs: Any, transformation: Any) -> None:
         """
         Parameters
